% PSHAPEDEC  Decompose an image on a vector basis of 2D polar shapelets
%     [PCOEFF RIMA] = PSHAPEDEC(IMA, nmax, beta) produces a vector PCOEFF
%     of (nmax+1)(nmax+2)/2 real polar shapelet coefficients, and a
%     reconstructed image RIMA from the input image IMA. beta is the
%     scaling factor of the polar shapelets.
%     (see Massey & Refregier 2004 astro-ph/0408445)
%
% Version 1.0 (03-18-2005)
%
% (C) E.Bertin <bertin@iap.fr> 2005
% Visit the EFIGI website http://www.efigi.org

function [pcoeff rima] = pshapedec(ima, nmax, beta);
width = size(ima,1);
height = size(ima,2);
p = pshapelets(width, height, nmax, beta);
% we reshape to simplify matrix computations
ntot = size(p,3);
p2 = reshape(p, width*height, ntot);
% normalize the shapelet basis vectors
norm = 1./diag(sqrt(abs(p2'*p2)));
p2 = repmat(norm', [width*height 1]).*p2;
% we reshape to simplify matrix computations
ima2 = reshape(ima, width*height,1);
pcoeff = p2'*ima2;
rima2 = p2*pcoeff;
rima = reshape(rima2, width, height);
