% PSHAPELETS  Create a data-cube of real 2D polar shapelet basis vectors.
%     P = PSHAPELETS(width,height, nmax, beta) creates of data-cube
%     of real 2D polar shapelet basis vectors with scale beta, up to order
%     nmax. Each of the (nmax+1)(nmax+2)/2 vectors has width x height
%     pixels.
%     (see Massey & Refregier 2004 astro-ph/0408445)
%
%     Note:
%     The vectors are not yet properly normalized.
%
% Version 1.0 (03-18-2005)
%
% (C) E.Bertin <bertin@iap.fr> 2005
% Visit the EFIGI website http://www.efigi.org

function cube=pshapelets(width,height, nmax,beta)

% Not quite right for even sizes
x0 = width/2;
y0 = height/2;

% generate x and y maps
if width<height
    rmax = width/2;
else
    rmax = height/2;
end

x = 1:width;
x = x-rmax;
x = repmat(x',[1 height]);
y = 1:height;
y = repmat(y,[width 1]);
y = y-rmax;

% generate r and theta maps
r = sqrt(x.^2+y.^2)/beta;
rmax = rmax / beta;
theta = atan2(y,x);

% generate basis vector planes
k = 1;
for n=0:int32(nmax),
    for m=(mod(n,2)):2:n,
        am = abs(m);
        nmam2 = double((n-am)/2);
        r2 = r.^2;
        cube(:,:,uint32(k)) = (-1).^nmam2./beta.^(double(am)+1).*sqrt(factorial(double(nmam2))./(pi*factorial(double((n+am)/2)))).*(r<=rmax).*r.^double(am).*laguerre(r2, nmam2, am).*exp(-r2/2).*cos(double(m).*theta);
        k = k + 1;
% keep the imaginary part only if m!=0
        if (m~=0)
            cube(:,:,uint32(k)) = (-1).^nmam2./beta.^(double(am)+1).*sqrt(factorial(double(nmam2))./(pi*factorial(double((n+am)/2)))).*(r<=rmax).*r.^double(am).*laguerre(r2, nmam2, am).*exp(-r2/2).*sin(double(m).*theta);
            k = k + 1;
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute Laguerre polynomial value at x laguerre(x,p,q)
function l=laguerre(x,p,q)
q = double(q);
if p==0
    l = 1;
elseif p==1
    l = 1 - x + q;
else
    lpm2 = 1;
    lpm1 = 1 - x + q;
    for n=2:p,
        l = (2+(q-1-x)./double(n)).*lpm1 - (1+(q-1)./double(n)).*lpm2;
        lpm2 = lpm1;
        lpm1 = l;
    end
end
